#ifndef INC_PLATFORM_H
#define INC_PLATFORM_H

/*
 * Serial port support
 */

// return code on success
#define result_serial_ok 0

// return code on generic error
#define result_serial_error -1

// return code on device lost
#define result_serial_nodevice -2

// return code on timeout
#define result_serial_timeout -3

handle_t open_port (const char* name, unsigned int timeout);

int close_port (handle_t handle);

int command_port_flush (handle_t handle);

int command_port_send (handle_t handle, const byte* command, size_t command_len);

int command_port_receive (handle_t handle, byte* response, size_t response_len);

/*
 * Threading support
 */

/* Callback for user actions */
typedef void (*fork_join_thread_function_t)(void *arg);

/* Platform-specific fork/join function */
result_t fork_join (fork_join_thread_function_t function, int count, void* args, size_t arg_element_size);

unsigned long long get_thread_id();

/*
 * Device enumeration support
 */

/* Callback for user actions */
typedef void (*enumerate_devices_directory_callback_t) (char* name, void* arg);

/* Platform-specific enumerator */
result_t enumerate_devices_directory (enumerate_devices_directory_callback_t callback, void* arg, int flags);

bool is_same_device (const char* name1, const char* name2);

/*
 * Error handling
 */
unsigned int get_system_error_code ();

wchar_t* get_system_error_str (int code);

void free_system_error_str (wchar_t* str);

/*
 * Misc
 */
int fix_usbser_sys(const char* device_name);

void XIMC_API msec_sleep(unsigned int msec);

void get_wallclock(time_t* sec, int* msec);

/*
 * Mutex
 */
typedef struct mutex_t mutex_t;

mutex_t* mutex_init(unsigned int nonce);
void mutex_close(mutex_t* mutex);
void mutex_lock(mutex_t* mutex);
void mutex_unlock(mutex_t* mutex);


#endif
